"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .mooverror import MoovError
from typing import Any, TYPE_CHECKING

from moovio_sdk.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .accountterminalapplicationerror import (
        AccountTerminalApplicationError,
        AccountTerminalApplicationErrorData,
    )
    from .addcapabilitieserror import AddCapabilitiesError, AddCapabilitiesErrorData
    from .apierror import APIError
    from .assigncountrieserror import AssignCountriesError, AssignCountriesErrorData
    from .authtokenrequesterror import AuthTokenRequestError, AuthTokenRequestErrorData
    from .bankaccountvalidationerror import (
        BankAccountValidationError,
        BankAccountValidationErrorData,
    )
    from .brandvalidationerror import BrandValidationError, BrandValidationErrorData
    from .cardacquiringrefund import CardAcquiringRefund, CardAcquiringRefundData
    from .connectaccountrequestvalidationerror import (
        ConnectAccountRequestValidationError,
        ConnectAccountRequestValidationErrorData,
    )
    from .createaccounterror import CreateAccountError, CreateAccountErrorData
    from .createinvoiceerror import CreateInvoiceError, CreateInvoiceErrorData
    from .createinvoicepaymenterror import (
        CreateInvoicePaymentError,
        CreateInvoicePaymentErrorData,
    )
    from .createpaymentlinkerror import (
        CreatePaymentLinkError,
        CreatePaymentLinkErrorData,
    )
    from .createsweepconfigerror import (
        CreateSweepConfigError,
        CreateSweepConfigErrorData,
    )
    from .createticketerror import CreateTicketError, CreateTicketErrorData
    from .createwalletvalidationerror import (
        CreateWalletValidationError,
        CreateWalletValidationErrorData,
    )
    from .createwebhookvalidationerror import (
        CreateWebhookValidationError,
        CreateWebhookValidationErrorData,
    )
    from .duplicatecarderror import DuplicateCardError, DuplicateCardErrorData
    from .feeplanagreementerror import FeePlanAgreementError, FeePlanAgreementErrorData
    from .fileuploadvalidationerror import (
        File,
        FileTypedDict,
        FileUploadValidationError,
        FileUploadValidationErrorData,
    )
    from .filevalidationerror import FileValidationError, FileValidationErrorData
    from .genericerror import GenericError, GenericErrorData
    from .imagemetadatavalidationerror import (
        ImageMetadataValidationError,
        ImageMetadataValidationErrorData,
    )
    from .imagerequestvalidationerror import (
        ImageRequestValidationError,
        ImageRequestValidationErrorData,
    )
    from .linkapplepayerror import LinkApplePayError, LinkApplePayErrorData
    from .linkcarderror import LinkCardError, LinkCardErrorData
    from .listinvoicesvalidationerror import (
        ListInvoicesValidationError,
        ListInvoicesValidationErrorData,
    )
    from .listtransfersvalidationerror import (
        ListTransfersValidationError,
        ListTransfersValidationErrorData,
    )
    from .listwalletsvalidationerror import (
        ListWalletsValidationError,
        ListWalletsValidationErrorData,
    )
    from .listwallettransactionsvalidationerror import (
        ListWalletTransactionsValidationError,
        ListWalletTransactionsValidationErrorData,
    )
    from .microdepositvalidationerror import (
        MicroDepositValidationError,
        MicroDepositValidationErrorData,
    )
    from .no_response_error import NoResponseError
    from .onboardinginviteerror import OnboardingInviteError, OnboardingInviteErrorData
    from .patchaccounterror import PatchAccountError, PatchAccountErrorData
    from .patchsweepconfigerror import PatchSweepConfigError, PatchSweepConfigErrorData
    from .patchtransfervalidationerror import (
        PatchTransferValidationError,
        PatchTransferValidationErrorData,
    )
    from .patchwalletvalidationerror import (
        PatchWalletValidationError,
        PatchWalletValidationErrorData,
    )
    from .productrequestvalidationerror import (
        ProductRequestValidationError,
        ProductRequestValidationErrorData,
    )
    from .refundvalidationerror import RefundValidationError, RefundValidationErrorData
    from .representativevalidationerror import (
        Error,
        ErrorTypedDict,
        RepresentativeValidationError,
        RepresentativeValidationErrorData,
    )
    from .requestcarderror import RequestCardError, RequestCardErrorData
    from .responsevalidationerror import ResponseValidationError
    from .reversalvalidationerror import (
        ReversalValidationError,
        ReversalValidationErrorData,
    )
    from .revoketokenrequesterror import (
        RevokeTokenRequestError,
        RevokeTokenRequestErrorData,
    )
    from .schedulevalidationerror import (
        ScheduleValidationError,
        ScheduleValidationErrorData,
    )
    from .terminalapplicationerror import (
        TerminalApplicationError,
        TerminalApplicationErrorData,
    )
    from .transfer import Transfer, TransferData
    from .transferoptionsvalidationerror import (
        TransferOptionsValidationError,
        TransferOptionsValidationErrorData,
    )
    from .transfervalidationerror import (
        TransferValidationError,
        TransferValidationErrorData,
    )
    from .updatecarderror import UpdateCardError, UpdateCardErrorData
    from .updateinvoiceerror import UpdateInvoiceError, UpdateInvoiceErrorData
    from .updateissuedcarderror import UpdateIssuedCardError, UpdateIssuedCardErrorData
    from .updatepaymentlinkerror import (
        UpdatePaymentLinkError,
        UpdatePaymentLinkErrorData,
    )
    from .updateticketerror import UpdateTicketError, UpdateTicketErrorData
    from .updateunderwritingerror import (
        UpdateUnderwritingError,
        UpdateUnderwritingErrorData,
        UpdateUnderwritingErrorError,
        UpdateUnderwritingErrorErrorTypedDict,
    )
    from .updatewebhookvalidationerror import (
        UpdateWebhookValidationError,
        UpdateWebhookValidationErrorData,
    )
    from .upsertunderwritingerror import (
        UpsertUnderwritingError,
        UpsertUnderwritingErrorData,
    )

__all__ = [
    "APIError",
    "AccountTerminalApplicationError",
    "AccountTerminalApplicationErrorData",
    "AddCapabilitiesError",
    "AddCapabilitiesErrorData",
    "AssignCountriesError",
    "AssignCountriesErrorData",
    "AuthTokenRequestError",
    "AuthTokenRequestErrorData",
    "BankAccountValidationError",
    "BankAccountValidationErrorData",
    "BrandValidationError",
    "BrandValidationErrorData",
    "CardAcquiringRefund",
    "CardAcquiringRefundData",
    "ConnectAccountRequestValidationError",
    "ConnectAccountRequestValidationErrorData",
    "CreateAccountError",
    "CreateAccountErrorData",
    "CreateInvoiceError",
    "CreateInvoiceErrorData",
    "CreateInvoicePaymentError",
    "CreateInvoicePaymentErrorData",
    "CreatePaymentLinkError",
    "CreatePaymentLinkErrorData",
    "CreateSweepConfigError",
    "CreateSweepConfigErrorData",
    "CreateTicketError",
    "CreateTicketErrorData",
    "CreateWalletValidationError",
    "CreateWalletValidationErrorData",
    "CreateWebhookValidationError",
    "CreateWebhookValidationErrorData",
    "DuplicateCardError",
    "DuplicateCardErrorData",
    "Error",
    "ErrorTypedDict",
    "FeePlanAgreementError",
    "FeePlanAgreementErrorData",
    "File",
    "FileTypedDict",
    "FileUploadValidationError",
    "FileUploadValidationErrorData",
    "FileValidationError",
    "FileValidationErrorData",
    "GenericError",
    "GenericErrorData",
    "ImageMetadataValidationError",
    "ImageMetadataValidationErrorData",
    "ImageRequestValidationError",
    "ImageRequestValidationErrorData",
    "LinkApplePayError",
    "LinkApplePayErrorData",
    "LinkCardError",
    "LinkCardErrorData",
    "ListInvoicesValidationError",
    "ListInvoicesValidationErrorData",
    "ListTransfersValidationError",
    "ListTransfersValidationErrorData",
    "ListWalletTransactionsValidationError",
    "ListWalletTransactionsValidationErrorData",
    "ListWalletsValidationError",
    "ListWalletsValidationErrorData",
    "MicroDepositValidationError",
    "MicroDepositValidationErrorData",
    "MoovError",
    "NoResponseError",
    "OnboardingInviteError",
    "OnboardingInviteErrorData",
    "PatchAccountError",
    "PatchAccountErrorData",
    "PatchSweepConfigError",
    "PatchSweepConfigErrorData",
    "PatchTransferValidationError",
    "PatchTransferValidationErrorData",
    "PatchWalletValidationError",
    "PatchWalletValidationErrorData",
    "ProductRequestValidationError",
    "ProductRequestValidationErrorData",
    "RefundValidationError",
    "RefundValidationErrorData",
    "RepresentativeValidationError",
    "RepresentativeValidationErrorData",
    "RequestCardError",
    "RequestCardErrorData",
    "ResponseValidationError",
    "ReversalValidationError",
    "ReversalValidationErrorData",
    "RevokeTokenRequestError",
    "RevokeTokenRequestErrorData",
    "ScheduleValidationError",
    "ScheduleValidationErrorData",
    "TerminalApplicationError",
    "TerminalApplicationErrorData",
    "Transfer",
    "TransferData",
    "TransferOptionsValidationError",
    "TransferOptionsValidationErrorData",
    "TransferValidationError",
    "TransferValidationErrorData",
    "UpdateCardError",
    "UpdateCardErrorData",
    "UpdateInvoiceError",
    "UpdateInvoiceErrorData",
    "UpdateIssuedCardError",
    "UpdateIssuedCardErrorData",
    "UpdatePaymentLinkError",
    "UpdatePaymentLinkErrorData",
    "UpdateTicketError",
    "UpdateTicketErrorData",
    "UpdateUnderwritingError",
    "UpdateUnderwritingErrorData",
    "UpdateUnderwritingErrorError",
    "UpdateUnderwritingErrorErrorTypedDict",
    "UpdateWebhookValidationError",
    "UpdateWebhookValidationErrorData",
    "UpsertUnderwritingError",
    "UpsertUnderwritingErrorData",
]

_dynamic_imports: dict[str, str] = {
    "AccountTerminalApplicationError": ".accountterminalapplicationerror",
    "AccountTerminalApplicationErrorData": ".accountterminalapplicationerror",
    "AddCapabilitiesError": ".addcapabilitieserror",
    "AddCapabilitiesErrorData": ".addcapabilitieserror",
    "APIError": ".apierror",
    "AssignCountriesError": ".assigncountrieserror",
    "AssignCountriesErrorData": ".assigncountrieserror",
    "AuthTokenRequestError": ".authtokenrequesterror",
    "AuthTokenRequestErrorData": ".authtokenrequesterror",
    "BankAccountValidationError": ".bankaccountvalidationerror",
    "BankAccountValidationErrorData": ".bankaccountvalidationerror",
    "BrandValidationError": ".brandvalidationerror",
    "BrandValidationErrorData": ".brandvalidationerror",
    "CardAcquiringRefund": ".cardacquiringrefund",
    "CardAcquiringRefundData": ".cardacquiringrefund",
    "ConnectAccountRequestValidationError": ".connectaccountrequestvalidationerror",
    "ConnectAccountRequestValidationErrorData": ".connectaccountrequestvalidationerror",
    "CreateAccountError": ".createaccounterror",
    "CreateAccountErrorData": ".createaccounterror",
    "CreateInvoiceError": ".createinvoiceerror",
    "CreateInvoiceErrorData": ".createinvoiceerror",
    "CreateInvoicePaymentError": ".createinvoicepaymenterror",
    "CreateInvoicePaymentErrorData": ".createinvoicepaymenterror",
    "CreatePaymentLinkError": ".createpaymentlinkerror",
    "CreatePaymentLinkErrorData": ".createpaymentlinkerror",
    "CreateSweepConfigError": ".createsweepconfigerror",
    "CreateSweepConfigErrorData": ".createsweepconfigerror",
    "CreateTicketError": ".createticketerror",
    "CreateTicketErrorData": ".createticketerror",
    "CreateWalletValidationError": ".createwalletvalidationerror",
    "CreateWalletValidationErrorData": ".createwalletvalidationerror",
    "CreateWebhookValidationError": ".createwebhookvalidationerror",
    "CreateWebhookValidationErrorData": ".createwebhookvalidationerror",
    "DuplicateCardError": ".duplicatecarderror",
    "DuplicateCardErrorData": ".duplicatecarderror",
    "FeePlanAgreementError": ".feeplanagreementerror",
    "FeePlanAgreementErrorData": ".feeplanagreementerror",
    "File": ".fileuploadvalidationerror",
    "FileTypedDict": ".fileuploadvalidationerror",
    "FileUploadValidationError": ".fileuploadvalidationerror",
    "FileUploadValidationErrorData": ".fileuploadvalidationerror",
    "FileValidationError": ".filevalidationerror",
    "FileValidationErrorData": ".filevalidationerror",
    "GenericError": ".genericerror",
    "GenericErrorData": ".genericerror",
    "ImageMetadataValidationError": ".imagemetadatavalidationerror",
    "ImageMetadataValidationErrorData": ".imagemetadatavalidationerror",
    "ImageRequestValidationError": ".imagerequestvalidationerror",
    "ImageRequestValidationErrorData": ".imagerequestvalidationerror",
    "LinkApplePayError": ".linkapplepayerror",
    "LinkApplePayErrorData": ".linkapplepayerror",
    "LinkCardError": ".linkcarderror",
    "LinkCardErrorData": ".linkcarderror",
    "ListInvoicesValidationError": ".listinvoicesvalidationerror",
    "ListInvoicesValidationErrorData": ".listinvoicesvalidationerror",
    "ListTransfersValidationError": ".listtransfersvalidationerror",
    "ListTransfersValidationErrorData": ".listtransfersvalidationerror",
    "ListWalletsValidationError": ".listwalletsvalidationerror",
    "ListWalletsValidationErrorData": ".listwalletsvalidationerror",
    "ListWalletTransactionsValidationError": ".listwallettransactionsvalidationerror",
    "ListWalletTransactionsValidationErrorData": ".listwallettransactionsvalidationerror",
    "MicroDepositValidationError": ".microdepositvalidationerror",
    "MicroDepositValidationErrorData": ".microdepositvalidationerror",
    "NoResponseError": ".no_response_error",
    "OnboardingInviteError": ".onboardinginviteerror",
    "OnboardingInviteErrorData": ".onboardinginviteerror",
    "PatchAccountError": ".patchaccounterror",
    "PatchAccountErrorData": ".patchaccounterror",
    "PatchSweepConfigError": ".patchsweepconfigerror",
    "PatchSweepConfigErrorData": ".patchsweepconfigerror",
    "PatchTransferValidationError": ".patchtransfervalidationerror",
    "PatchTransferValidationErrorData": ".patchtransfervalidationerror",
    "PatchWalletValidationError": ".patchwalletvalidationerror",
    "PatchWalletValidationErrorData": ".patchwalletvalidationerror",
    "ProductRequestValidationError": ".productrequestvalidationerror",
    "ProductRequestValidationErrorData": ".productrequestvalidationerror",
    "RefundValidationError": ".refundvalidationerror",
    "RefundValidationErrorData": ".refundvalidationerror",
    "Error": ".representativevalidationerror",
    "ErrorTypedDict": ".representativevalidationerror",
    "RepresentativeValidationError": ".representativevalidationerror",
    "RepresentativeValidationErrorData": ".representativevalidationerror",
    "RequestCardError": ".requestcarderror",
    "RequestCardErrorData": ".requestcarderror",
    "ResponseValidationError": ".responsevalidationerror",
    "ReversalValidationError": ".reversalvalidationerror",
    "ReversalValidationErrorData": ".reversalvalidationerror",
    "RevokeTokenRequestError": ".revoketokenrequesterror",
    "RevokeTokenRequestErrorData": ".revoketokenrequesterror",
    "ScheduleValidationError": ".schedulevalidationerror",
    "ScheduleValidationErrorData": ".schedulevalidationerror",
    "TerminalApplicationError": ".terminalapplicationerror",
    "TerminalApplicationErrorData": ".terminalapplicationerror",
    "Transfer": ".transfer",
    "TransferData": ".transfer",
    "TransferOptionsValidationError": ".transferoptionsvalidationerror",
    "TransferOptionsValidationErrorData": ".transferoptionsvalidationerror",
    "TransferValidationError": ".transfervalidationerror",
    "TransferValidationErrorData": ".transfervalidationerror",
    "UpdateCardError": ".updatecarderror",
    "UpdateCardErrorData": ".updatecarderror",
    "UpdateInvoiceError": ".updateinvoiceerror",
    "UpdateInvoiceErrorData": ".updateinvoiceerror",
    "UpdateIssuedCardError": ".updateissuedcarderror",
    "UpdateIssuedCardErrorData": ".updateissuedcarderror",
    "UpdatePaymentLinkError": ".updatepaymentlinkerror",
    "UpdatePaymentLinkErrorData": ".updatepaymentlinkerror",
    "UpdateTicketError": ".updateticketerror",
    "UpdateTicketErrorData": ".updateticketerror",
    "UpdateUnderwritingError": ".updateunderwritingerror",
    "UpdateUnderwritingErrorData": ".updateunderwritingerror",
    "UpdateUnderwritingErrorError": ".updateunderwritingerror",
    "UpdateUnderwritingErrorErrorTypedDict": ".updateunderwritingerror",
    "UpdateWebhookValidationError": ".updatewebhookvalidationerror",
    "UpdateWebhookValidationErrorData": ".updatewebhookvalidationerror",
    "UpsertUnderwritingError": ".upsertunderwritingerror",
    "UpsertUnderwritingErrorData": ".upsertunderwritingerror",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addresssuggestion import AddressSuggestion, AddressSuggestionTypedDict
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class EnrichedAddressResponseTypedDict(TypedDict):
    suggestions: List[AddressSuggestionTypedDict]


class EnrichedAddressResponse(BaseModel):
    suggestions: List[AddressSuggestion]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Scheduling(BaseSDK):
    def create(
        self,
        *,
        account_id: str,
        description: Optional[str] = None,
        occurrences: Optional[
            Union[List[components.Occurrence], List[components.OccurrenceTypedDict]]
        ] = None,
        recur: Optional[Union[components.Recur, components.RecurTypedDict]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateScheduleResponse:
        r"""Describes the schedule to create or modify.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Account ID of the account that will run the transfer.
        :param description: Simple description of what the schedule is.
        :param occurrences:
        :param recur: Defines configuration for recurring transfers.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateScheduleRequest(
            account_id=account_id,
            upsert_schedule=components.UpsertSchedule(
                description=description,
                occurrences=utils.get_pydantic_model(
                    occurrences, Optional[List[components.Occurrence]]
                ),
                recur=utils.get_pydantic_model(recur, Optional[components.Recur]),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_schedule, False, False, "json", components.UpsertSchedule
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateScheduleResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ScheduleValidationErrorData, http_res
            )
            raise errors.ScheduleValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        account_id: str,
        description: Optional[str] = None,
        occurrences: Optional[
            Union[List[components.Occurrence], List[components.OccurrenceTypedDict]]
        ] = None,
        recur: Optional[Union[components.Recur, components.RecurTypedDict]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateScheduleResponse:
        r"""Describes the schedule to create or modify.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Account ID of the account that will run the transfer.
        :param description: Simple description of what the schedule is.
        :param occurrences:
        :param recur: Defines configuration for recurring transfers.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateScheduleRequest(
            account_id=account_id,
            upsert_schedule=components.UpsertSchedule(
                description=description,
                occurrences=utils.get_pydantic_model(
                    occurrences, Optional[List[components.Occurrence]]
                ),
                recur=utils.get_pydantic_model(recur, Optional[components.Recur]),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_schedule, False, False, "json", components.UpsertSchedule
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateScheduleResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ScheduleValidationErrorData, http_res
            )
            raise errors.ScheduleValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        hydrate: Optional[operations.Hydrate] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListSchedulesResponse:
        r"""Describes a list of schedules associated with an account. Append the `hydrate=accounts` query parameter to include partial account details in the response.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param skip:
        :param count:
        :param hydrate:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListSchedulesRequest(
            skip=skip,
            count=count,
            hydrate=hydrate,
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListSchedulesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listSchedules",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListSchedulesResponse(
                result=unmarshal_json_response(
                    List[components.ScheduleListResponse], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        hydrate: Optional[operations.Hydrate] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListSchedulesResponse:
        r"""Describes a list of schedules associated with an account. Append the `hydrate=accounts` query parameter to include partial account details in the response.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param skip:
        :param count:
        :param hydrate:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListSchedulesRequest(
            skip=skip,
            count=count,
            hydrate=hydrate,
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListSchedulesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listSchedules",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListSchedulesResponse(
                result=unmarshal_json_response(
                    List[components.ScheduleListResponse], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        account_id: str,
        schedule_id: str,
        description: Optional[str] = None,
        occurrences: Optional[
            Union[List[components.Occurrence], List[components.OccurrenceTypedDict]]
        ] = None,
        recur: Optional[Union[components.Recur, components.RecurTypedDict]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateScheduleResponse:
        r"""Describes the schedule to modify.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Account ID of the account that will run the transfer.
        :param schedule_id:
        :param description: Simple description of what the schedule is.
        :param occurrences:
        :param recur: Defines configuration for recurring transfers.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateScheduleRequest(
            account_id=account_id,
            schedule_id=schedule_id,
            upsert_schedule=components.UpsertSchedule(
                description=description,
                occurrences=utils.get_pydantic_model(
                    occurrences, Optional[List[components.Occurrence]]
                ),
                recur=utils.get_pydantic_model(recur, Optional[components.Recur]),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_schedule, False, False, "json", components.UpsertSchedule
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateScheduleResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ScheduleValidationErrorData, http_res
            )
            raise errors.ScheduleValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        account_id: str,
        schedule_id: str,
        description: Optional[str] = None,
        occurrences: Optional[
            Union[List[components.Occurrence], List[components.OccurrenceTypedDict]]
        ] = None,
        recur: Optional[Union[components.Recur, components.RecurTypedDict]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateScheduleResponse:
        r"""Describes the schedule to modify.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Account ID of the account that will run the transfer.
        :param schedule_id:
        :param description: Simple description of what the schedule is.
        :param occurrences:
        :param recur: Defines configuration for recurring transfers.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateScheduleRequest(
            account_id=account_id,
            schedule_id=schedule_id,
            upsert_schedule=components.UpsertSchedule(
                description=description,
                occurrences=utils.get_pydantic_model(
                    occurrences, Optional[List[components.Occurrence]]
                ),
                recur=utils.get_pydantic_model(recur, Optional[components.Recur]),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_schedule, False, False, "json", components.UpsertSchedule
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateScheduleResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ScheduleValidationErrorData, http_res
            )
            raise errors.ScheduleValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetSchedulesResponse:
        r"""Describes a schedule associated with an account. Requires at least 1 occurrence or recurTransfer to be specified.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetSchedulesRequest(
            account_id=account_id,
            schedule_id=schedule_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetSchedulesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getSchedules",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetSchedulesResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetSchedulesResponse:
        r"""Describes a schedule associated with an account. Requires at least 1 occurrence or recurTransfer to be specified.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetSchedulesRequest(
            account_id=account_id,
            schedule_id=schedule_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetSchedulesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getSchedules",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetSchedulesResponse(
                result=unmarshal_json_response(components.ScheduleResponse, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def cancel(
        self,
        *,
        account_id: str,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CancelScheduleResponse:
        r"""Describes the schedule to cancel.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Your Moov account ID as the partner running the transfers.
        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CancelScheduleRequest(
            account_id=account_id,
            schedule_id=schedule_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CancelScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="cancelSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.CancelScheduleResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def cancel_async(
        self,
        *,
        account_id: str,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CancelScheduleResponse:
        r"""Describes the schedule to cancel.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: Your Moov account ID as the partner running the transfers.
        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CancelScheduleRequest(
            account_id=account_id,
            schedule_id=schedule_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/accounts/{accountID}/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CancelScheduleGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="cancelSchedule",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.CancelScheduleResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_occurrance(
        self,
        *,
        account_id: str,
        schedule_id: str,
        occurrence_filter: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetScheduledOccurrenceResponse:
        r"""Gets a specific occurrence.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param schedule_id:
        :param occurrence_filter: Allows the specification of additional filters beyond the string.

            Specifying a string returns the exact occurrence.
            Specifying a RFC 3339 timestamp returns the latest occurrence at or before that timestamp.
            Specifying `latest` returns the latest occurrence at or before now.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetScheduledOccurrenceRequest(
            account_id=account_id,
            schedule_id=schedule_id,
            occurrence_filter=occurrence_filter,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/schedules/{scheduleID}/occurrences/{occurrenceFilter}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetScheduledOccurrenceGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getScheduledOccurrence",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetScheduledOccurrenceResponse(
                result=unmarshal_json_response(
                    components.OccurrencesResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_occurrance_async(
        self,
        *,
        account_id: str,
        schedule_id: str,
        occurrence_filter: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetScheduledOccurrenceResponse:
        r"""Gets a specific occurrence.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id:
        :param schedule_id:
        :param occurrence_filter: Allows the specification of additional filters beyond the string.

            Specifying a string returns the exact occurrence.
            Specifying a RFC 3339 timestamp returns the latest occurrence at or before that timestamp.
            Specifying `latest` returns the latest occurrence at or before now.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetScheduledOccurrenceRequest(
            account_id=account_id,
            schedule_id=schedule_id,
            occurrence_filter=occurrence_filter,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/schedules/{scheduleID}/occurrences/{occurrenceFilter}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetScheduledOccurrenceGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getScheduledOccurrence",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetScheduledOccurrenceResponse(
                result=unmarshal_json_response(
                    components.OccurrencesResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .receiptkind import ReceiptKind
from .sentreceipt import SentReceipt, SentReceiptTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ReceiptResponseTypedDict(TypedDict):
    receipt_id: str
    r"""Unique identifier for the receipt request."""
    created_by: str
    r"""AccountID for which the receipt request was created."""
    kind: ReceiptKind
    r"""The type of receipt."""
    disabled_on: NotRequired[datetime]
    r"""The date and time the receipt was disabled."""
    email: NotRequired[str]
    r"""The email address the receipt is sent to.
    Either email or emailAccountID will be in the response, but not both.
    """
    email_account_id: NotRequired[str]
    r"""The accountID the receipt is sent to.
    Either email or emailAccountID will be in the response, but not both.
    """
    for_transfer_id: NotRequired[str]
    r"""The ID of the transfer associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """
    for_schedule_id: NotRequired[str]
    r"""The ID of the schedule associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """
    for_occurrence_id: NotRequired[str]
    r"""The ID of the schedule occurrence associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """
    sent_for: NotRequired[List[SentReceiptTypedDict]]
    r"""The list of receipts that have been sent."""


class ReceiptResponse(BaseModel):
    receipt_id: Annotated[str, pydantic.Field(alias="receiptID")]
    r"""Unique identifier for the receipt request."""

    created_by: Annotated[str, pydantic.Field(alias="createdBy")]
    r"""AccountID for which the receipt request was created."""

    kind: ReceiptKind
    r"""The type of receipt."""

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )
    r"""The date and time the receipt was disabled."""

    email: Optional[str] = None
    r"""The email address the receipt is sent to.
    Either email or emailAccountID will be in the response, but not both.
    """

    email_account_id: Annotated[
        Optional[str], pydantic.Field(alias="emailAccountID")
    ] = None
    r"""The accountID the receipt is sent to.
    Either email or emailAccountID will be in the response, but not both.
    """

    for_transfer_id: Annotated[Optional[str], pydantic.Field(alias="forTransferID")] = (
        None
    )
    r"""The ID of the transfer associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """

    for_schedule_id: Annotated[Optional[str], pydantic.Field(alias="forScheduleID")] = (
        None
    )
    r"""The ID of the schedule associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """

    for_occurrence_id: Annotated[
        Optional[str], pydantic.Field(alias="forOccurrenceID")
    ] = None
    r"""The ID of the schedule occurrence associated with this receipt.
    Exactly one of forTransferID, forScheduleID, or forOccurrenceID must be provided.
    """

    sent_for: Annotated[
        Optional[List[SentReceipt]], pydantic.Field(alias="sentFor")
    ] = None
    r"""The list of receipts that have been sent."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "disabledOn",
                "email",
                "emailAccountID",
                "forTransferID",
                "forScheduleID",
                "forOccurrenceID",
                "sentFor",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ReceiptResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .sourcedestinationoptions import (
    SourceDestinationOptions,
    SourceDestinationOptionsTypedDict,
)
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CreateTransferOptionsTypedDict(TypedDict):
    source: SourceDestinationOptionsTypedDict
    destination: SourceDestinationOptionsTypedDict
    amount: AmountTypedDict


class CreateTransferOptions(BaseModel):
    source: SourceDestinationOptions

    destination: SourceDestinationOptions

    amount: Amount



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AccountTerminalApplicationTypedDict(TypedDict):
    r"""Describes link between an account and a terminal application."""

    account_id: str
    r"""ID of the merchant account."""
    terminal_application_id: str
    r"""ID of the terminal application."""


class AccountTerminalApplication(BaseModel):
    r"""Describes link between an account and a terminal application."""

    account_id: Annotated[str, pydantic.Field(alias="accountID")]
    r"""ID of the merchant account."""

    terminal_application_id: Annotated[
        str, pydantic.Field(alias="terminalApplicationID")
    ]
    r"""ID of the terminal application."""


try:
    AccountTerminalApplication.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .birthdate import BirthDate, BirthDateTypedDict
from .governmentid import GovernmentID, GovernmentIDTypedDict
from .individualname import IndividualName, IndividualNameTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .representativeresponsibilities import (
    RepresentativeResponsibilities,
    RepresentativeResponsibilitiesTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateRepresentativeTypedDict(TypedDict):
    r"""Describes a business representative to create."""

    name: IndividualNameTypedDict
    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]
    address: NotRequired[AddressTypedDict]
    birth_date: NotRequired[BirthDateTypedDict]
    government_id: NotRequired[GovernmentIDTypedDict]
    responsibilities: NotRequired[RepresentativeResponsibilitiesTypedDict]
    r"""Describes the job responsibilities of a business representative."""


class CreateRepresentative(BaseModel):
    r"""Describes a business representative to create."""

    name: IndividualName

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    address: Optional[Address] = None

    birth_date: Annotated[Optional[BirthDate], pydantic.Field(alias="birthDate")] = None

    government_id: Annotated[
        Optional[GovernmentID], pydantic.Field(alias="governmentID")
    ] = None

    responsibilities: Optional[RepresentativeResponsibilities] = None
    r"""Describes the job responsibilities of a business representative."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "phone",
                "email",
                "address",
                "birthDate",
                "governmentID",
                "responsibilities",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateRepresentative.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CardExpirationTypedDict(TypedDict):
    r"""The expiration date of the card or token."""

    month: str
    year: str


class CardExpiration(BaseModel):
    r"""The expiration date of the card or token."""

    month: str

    year: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ResidualTypedDict(TypedDict):
    residual_id: str
    r"""Unique identifier for this residual payment calculation."""
    partner_account_id: str
    r"""The partner account ID this residual belongs to."""
    period_start: datetime
    r"""Start date and time of the residual calculation period."""
    period_end: datetime
    r"""End date and time of the residual calculation period."""
    merchant_fees: AmountDecimalTypedDict
    r"""Total amount of merchant fees collected during the period. This represents the partner's revenue from merchant fees."""
    partner_cost: AmountDecimalTypedDict
    r"""Partner's total cost (buy rate) during the period."""
    net_income: AmountDecimalTypedDict
    r"""Net income calculated as merchant fee revenue minus partner costs."""
    revenue_share: str
    r"""The revenue share percentage the partner receives, expressed as a decimal string (e.g., \"25.00\" for 25%)."""
    residual_amount: AmountDecimalTypedDict
    r"""The amount the partner receives as their share of the net income (netIncome × revenueShare)."""
    created_on: datetime
    r"""Timestamp when the residual was created."""
    updated_on: datetime
    r"""Timestamp when the residual was last updated."""


class Residual(BaseModel):
    residual_id: Annotated[str, pydantic.Field(alias="residualID")]
    r"""Unique identifier for this residual payment calculation."""

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]
    r"""The partner account ID this residual belongs to."""

    period_start: Annotated[datetime, pydantic.Field(alias="periodStart")]
    r"""Start date and time of the residual calculation period."""

    period_end: Annotated[datetime, pydantic.Field(alias="periodEnd")]
    r"""End date and time of the residual calculation period."""

    merchant_fees: Annotated[AmountDecimal, pydantic.Field(alias="merchantFees")]
    r"""Total amount of merchant fees collected during the period. This represents the partner's revenue from merchant fees."""

    partner_cost: Annotated[AmountDecimal, pydantic.Field(alias="partnerCost")]
    r"""Partner's total cost (buy rate) during the period."""

    net_income: Annotated[AmountDecimal, pydantic.Field(alias="netIncome")]
    r"""Net income calculated as merchant fee revenue minus partner costs."""

    revenue_share: Annotated[str, pydantic.Field(alias="revenueShare")]
    r"""The revenue share percentage the partner receives, expressed as a decimal string (e.g., \"25.00\" for 25%)."""

    residual_amount: Annotated[AmountDecimal, pydantic.Field(alias="residualAmount")]
    r"""The amount the partner receives as their share of the net income (netIncome × revenueShare)."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]
    r"""Timestamp when the residual was created."""

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]
    r"""Timestamp when the residual was last updated."""


try:
    Residual.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .industrytaxonomy import IndustryTaxonomy, IndustryTaxonomyTypedDict
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class EnrichedIndustriesTypedDict(TypedDict):
    industries: List[IndustryTaxonomyTypedDict]


class EnrichedIndustries(BaseModel):
    industries: List[IndustryTaxonomy]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createtransfersourceach import (
    CreateTransferSourceACH,
    CreateTransferSourceACHTypedDict,
)
from .createtransfersourcecard import (
    CreateTransferSourceCard,
    CreateTransferSourceCardTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferSourceTypedDict(TypedDict):
    r"""Where funds for a transfer originate. For the source, you must include either a `paymentMethodID` or a `transferID`."""

    transfer_id: NotRequired[str]
    r"""A `transferID` is used to create a [transfer group](https://docs.moov.io/guides/money-movement/transfer-groups/),
    associating the new transfer with a parent transfer.
    """
    payment_method_id: NotRequired[str]
    payment_token: NotRequired[str]
    card_details: NotRequired[CreateTransferSourceCardTypedDict]
    ach_details: NotRequired[CreateTransferSourceACHTypedDict]


class CreateTransferSource(BaseModel):
    r"""Where funds for a transfer originate. For the source, you must include either a `paymentMethodID` or a `transferID`."""

    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferID")] = None
    r"""A `transferID` is used to create a [transfer group](https://docs.moov.io/guides/money-movement/transfer-groups/),
    associating the new transfer with a parent transfer.
    """

    payment_method_id: Annotated[
        Optional[str], pydantic.Field(alias="paymentMethodID")
    ] = None

    payment_token: Annotated[Optional[str], pydantic.Field(alias="paymentToken")] = None

    card_details: Annotated[
        Optional[CreateTransferSourceCard], pydantic.Field(alias="cardDetails")
    ] = None

    ach_details: Annotated[
        Optional[CreateTransferSourceACH], pydantic.Field(alias="achDetails")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "transferID",
                "paymentMethodID",
                "paymentToken",
                "cardDetails",
                "achDetails",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateTransferSource.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .wallettype import WalletType
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class TransferPaymentMethodsWalletTypedDict(TypedDict):
    wallet_id: str
    partner_account_id: str
    wallet_type: WalletType
    r"""Type of a wallet.
    - `default`: The system-generated wallet automatically created when an account is granted the wallet capability.
    - `general`: An additional, user-defined wallet created via API or Dashboard.
    """


class TransferPaymentMethodsWallet(BaseModel):
    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]

    wallet_type: Annotated[WalletType, pydantic.Field(alias="walletType")]
    r"""Type of a wallet.
    - `default`: The system-generated wallet automatically created when an account is granted the wallet capability.
    - `general`: An additional, user-defined wallet created via API or Dashboard.
    """


try:
    TransferPaymentMethodsWallet.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class IndustryTaxonomyTypedDict(TypedDict):
    r"""A structured industry taxonomy entry with category and mapped to a default MCC code."""

    industry: str
    r"""Classification identifier for the industry."""
    display_name: str
    r"""Display name of the industry"""
    category: str
    r"""Category slug"""
    category_display_name: str
    r"""Human-readable category label"""
    default_mcc: str
    r"""Default Merchant Category Code"""


class IndustryTaxonomy(BaseModel):
    r"""A structured industry taxonomy entry with category and mapped to a default MCC code."""

    industry: str
    r"""Classification identifier for the industry."""

    display_name: Annotated[str, pydantic.Field(alias="displayName")]
    r"""Display name of the industry"""

    category: str
    r"""Category slug"""

    category_display_name: Annotated[str, pydantic.Field(alias="categoryDisplayName")]
    r"""Human-readable category label"""

    default_mcc: Annotated[str, pydantic.Field(alias="defaultMcc")]
    r"""Default Merchant Category Code"""


try:
    IndustryTaxonomy.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addresserror import AddressError, AddressErrorTypedDict
from .birthdateerror import BirthDateError, BirthDateErrorTypedDict
from .individualnameerror import IndividualNameError, IndividualNameErrorTypedDict
from .phonenumbererror import PhoneNumberError, PhoneNumberErrorTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateIndividualErrorSsnTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class CreateIndividualErrorSsn(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateIndividualErrorItinTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class CreateIndividualErrorItin(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateIndividualErrorGovernmentIDTypedDict(TypedDict):
    ssn: NotRequired[CreateIndividualErrorSsnTypedDict]
    itin: NotRequired[CreateIndividualErrorItinTypedDict]


class CreateIndividualErrorGovernmentID(BaseModel):
    ssn: Optional[CreateIndividualErrorSsn] = None

    itin: Optional[CreateIndividualErrorItin] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ssn", "itin"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateIndividualErrorTypedDict(TypedDict):
    name: NotRequired[IndividualNameErrorTypedDict]
    phone: NotRequired[PhoneNumberErrorTypedDict]
    email: NotRequired[str]
    address: NotRequired[AddressErrorTypedDict]
    birth_date: NotRequired[BirthDateErrorTypedDict]
    government_id: NotRequired[CreateIndividualErrorGovernmentIDTypedDict]


class CreateIndividualError(BaseModel):
    name: Optional[IndividualNameError] = None

    phone: Optional[PhoneNumberError] = None

    email: Optional[str] = None

    address: Optional[AddressError] = None

    birth_date: Annotated[
        Optional[BirthDateError], pydantic.Field(alias="birthDate")
    ] = None

    government_id: Annotated[
        Optional[CreateIndividualErrorGovernmentID],
        pydantic.Field(alias="governmentID"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["name", "phone", "email", "address", "birthDate", "governmentID"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateIndividualErrorSsn.model_rebuild()
except NameError:
    pass
try:
    CreateIndividualErrorItin.model_rebuild()
except NameError:
    pass
try:
    CreateIndividualError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .sweepconfigpaymentmethod import (
    SweepConfigPaymentMethod,
    SweepConfigPaymentMethodTypedDict,
)
from .sweepconfigstatus import SweepConfigStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SweepConfigTypedDict(TypedDict):
    sweep_config_id: str
    wallet_id: str
    status: SweepConfigStatus
    push_payment_method: SweepConfigPaymentMethodTypedDict
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """
    pull_payment_method: SweepConfigPaymentMethodTypedDict
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """
    created_on: datetime
    updated_on: datetime
    minimum_balance: NotRequired[str]
    r"""An optional field to specify an amount to maintain in the wallet. This is a decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99. If not supplied, the default is 0.00."""
    statement_descriptor: NotRequired[str]
    r"""The text that appears on the banking statement. The default descriptor is a 10 character ID if an override is not set in the sweep configs statementDescriptor."""
    locked_fields: NotRequired[List[str]]
    r"""An array of fields that are locked. To request updates, please contact Moov support."""


class SweepConfig(BaseModel):
    sweep_config_id: Annotated[str, pydantic.Field(alias="sweepConfigID")]

    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    status: SweepConfigStatus

    push_payment_method: Annotated[
        SweepConfigPaymentMethod, pydantic.Field(alias="pushPaymentMethod")
    ]
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """

    pull_payment_method: Annotated[
        SweepConfigPaymentMethod, pydantic.Field(alias="pullPaymentMethod")
    ]
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    minimum_balance: Annotated[
        Optional[str], pydantic.Field(alias="minimumBalance")
    ] = None
    r"""An optional field to specify an amount to maintain in the wallet. This is a decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99. If not supplied, the default is 0.00."""

    statement_descriptor: Annotated[
        Optional[str], pydantic.Field(alias="statementDescriptor")
    ] = None
    r"""The text that appears on the banking statement. The default descriptor is a 10 character ID if an override is not set in the sweep configs statementDescriptor."""

    locked_fields: Annotated[
        Optional[List[str]], pydantic.Field(alias="lockedFields")
    ] = None
    r"""An array of fields that are locked. To request updates, please contact Moov support."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["minimumBalance", "statementDescriptor", "lockedFields"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SweepConfig.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountupdate import AmountUpdate, AmountUpdateTypedDict
from .createpaymentlinklineitemsupdate import (
    CreatePaymentLinkLineItemsUpdate,
    CreatePaymentLinkLineItemsUpdateTypedDict,
)
from .paymentlinkcustomeroptions import (
    PaymentLinkCustomerOptions,
    PaymentLinkCustomerOptionsTypedDict,
)
from .paymentlinkdisplayoptionsupdate import (
    PaymentLinkDisplayOptionsUpdate,
    PaymentLinkDisplayOptionsUpdateTypedDict,
)
from .paymentlinkpaymentdetailsupdate import (
    PaymentLinkPaymentDetailsUpdate,
    PaymentLinkPaymentDetailsUpdateTypedDict,
)
from .paymentlinkpayoutdetailsupdate import (
    PaymentLinkPayoutDetailsUpdate,
    PaymentLinkPayoutDetailsUpdateTypedDict,
)
from datetime import datetime
from moovio_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdatePaymentLinkTypedDict(TypedDict):
    amount: NotRequired[AmountUpdateTypedDict]
    sales_tax_amount: NotRequired[AmountUpdateTypedDict]
    expires_on: NotRequired[Nullable[datetime]]
    display: NotRequired[PaymentLinkDisplayOptionsUpdateTypedDict]
    r"""Customizable display options for a payment link."""
    customer: NotRequired[PaymentLinkCustomerOptionsTypedDict]
    payment: NotRequired[PaymentLinkPaymentDetailsUpdateTypedDict]
    r"""Options for payment links used to collect payment."""
    payout: NotRequired[PaymentLinkPayoutDetailsUpdateTypedDict]
    line_items: NotRequired[CreatePaymentLinkLineItemsUpdateTypedDict]
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """


class UpdatePaymentLink(BaseModel):
    amount: Optional[AmountUpdate] = None

    sales_tax_amount: Annotated[
        Optional[AmountUpdate], pydantic.Field(alias="salesTaxAmount")
    ] = None

    expires_on: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="expiresOn")
    ] = UNSET

    display: Optional[PaymentLinkDisplayOptionsUpdate] = None
    r"""Customizable display options for a payment link."""

    customer: Optional[PaymentLinkCustomerOptions] = None

    payment: Optional[PaymentLinkPaymentDetailsUpdate] = None
    r"""Options for payment links used to collect payment."""

    payout: Optional[PaymentLinkPayoutDetailsUpdate] = None

    line_items: Annotated[
        Optional[CreatePaymentLinkLineItemsUpdate], pydantic.Field(alias="lineItems")
    ] = None
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "salesTaxAmount",
                "expiresOn",
                "display",
                "customer",
                "payment",
                "payout",
                "lineItems",
            ]
        )
        nullable_fields = set(["expiresOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    UpdatePaymentLink.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .brandcolor import BrandColor, BrandColorTypedDict
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class BrandColorsTypedDict(TypedDict):
    r"""Brand colors for light and dark modes."""

    dark: BrandColorTypedDict
    light: BrandColorTypedDict


class BrandColors(BaseModel):
    r"""Brand colors for light and dark modes."""

    dark: BrandColor

    light: BrandColor



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubmissionIntent(str, Enum):
    WAIT = "wait"
    SUBMIT = "submit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .businesspresence import BusinessPresence
from .collectfunds import CollectFunds, CollectFundsTypedDict
from .geographicreach import GeographicReach
from .moneytransfer import MoneyTransfer, MoneyTransferTypedDict
from .pendinglitigation import PendingLitigation
from .sendfunds import SendFunds, SendFundsTypedDict
from .submissionintent import SubmissionIntent
from .volumesharebycustomertype import (
    VolumeShareByCustomerType,
    VolumeShareByCustomerTypeTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpsertUnderwritingTypedDict(TypedDict):
    geographic_reach: NotRequired[GeographicReach]
    business_presence: NotRequired[BusinessPresence]
    pending_litigation: NotRequired[PendingLitigation]
    volume_share_by_customer_type: NotRequired[VolumeShareByCustomerTypeTypedDict]
    collect_funds: NotRequired[CollectFundsTypedDict]
    money_transfer: NotRequired[MoneyTransferTypedDict]
    send_funds: NotRequired[SendFundsTypedDict]
    submission_intent: NotRequired[SubmissionIntent]


class UpsertUnderwriting(BaseModel):
    geographic_reach: Annotated[
        Optional[GeographicReach], pydantic.Field(alias="geographicReach")
    ] = None

    business_presence: Annotated[
        Optional[BusinessPresence], pydantic.Field(alias="businessPresence")
    ] = None

    pending_litigation: Annotated[
        Optional[PendingLitigation], pydantic.Field(alias="pendingLitigation")
    ] = None

    volume_share_by_customer_type: Annotated[
        Optional[VolumeShareByCustomerType],
        pydantic.Field(alias="volumeShareByCustomerType"),
    ] = None

    collect_funds: Annotated[
        Optional[CollectFunds], pydantic.Field(alias="collectFunds")
    ] = None

    money_transfer: Annotated[
        Optional[MoneyTransfer], pydantic.Field(alias="moneyTransfer")
    ] = None

    send_funds: Annotated[Optional[SendFunds], pydantic.Field(alias="sendFunds")] = None

    submission_intent: Annotated[
        Optional[SubmissionIntent], pydantic.Field(alias="submissionIntent")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "geographicReach",
                "businessPresence",
                "pendingLitigation",
                "volumeShareByCustomerType",
                "collectFunds",
                "moneyTransfer",
                "sendFunds",
                "submissionIntent",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    UpsertUnderwriting.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Mapping, Optional


class EnrichedAddress(BaseSDK):
    def get(
        self,
        *,
        search: str,
        max_results: Optional[int] = None,
        include_cities: Optional[str] = None,
        include_states: Optional[str] = None,
        include_zipcodes: Optional[str] = None,
        exclude_states: Optional[str] = None,
        prefer_cities: Optional[str] = None,
        prefer_states: Optional[str] = None,
        prefer_zipcodes: Optional[str] = None,
        prefer_ratio: Optional[int] = None,
        prefer_geolocation: Optional[str] = None,
        selected: Optional[str] = None,
        source: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetEnrichmentAddressResponse:
        r"""Fetch enriched address suggestions. Requires a partial address.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/profile-enrichment.read` scope.

        :param search: Partial or complete address to search.
        :param max_results: Maximum number of results to return.
        :param include_cities: Limits results to a list of given cities.
        :param include_states: Limits results to a list of given states.
        :param include_zipcodes: Limits results to a list of given zipcodes.
        :param exclude_states: Exclude list of states from results. No `include` pararmeters may be used with this parameter.
        :param prefer_cities: Display results with the listed cities at the top.
        :param prefer_states: Display results with the listed states at the top.
        :param prefer_zipcodes: Display results with the listed zipcodes at the top.
        :param prefer_ratio: Specifies the percentage of address suggestions that should be preferred and will appear at the top of the results.
        :param prefer_geolocation: If omitted or set to `city`, it uses the sender's IP address to determine location, then automatically adds the city and state
            to the preferCities value. This parameter takes precedence over other `include` or `exclude` parameters meaning that if it is
            not set to `none`, you may see addresses from areas you do not wish to see.
        :param selected: Useful for narrowing results with `addressLine2` suggestions such as `Apt` (denotes an apartment building with multiple residences).
        :param source: Include results from alternate data sources. Allowed values are `all` (non-postal addresses), or `postal` (postal addresses only).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetEnrichmentAddressRequest(
            search=search,
            max_results=max_results,
            include_cities=include_cities,
            include_states=include_states,
            include_zipcodes=include_zipcodes,
            exclude_states=exclude_states,
            prefer_cities=prefer_cities,
            prefer_states=prefer_states,
            prefer_zipcodes=prefer_zipcodes,
            prefer_ratio=prefer_ratio,
            prefer_geolocation=prefer_geolocation,
            selected=selected,
            source=source,
        )

        req = self._build_request(
            method="GET",
            path="/enrichment/address",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetEnrichmentAddressGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getEnrichmentAddress",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetEnrichmentAddressResponse(
                result=unmarshal_json_response(
                    components.EnrichedAddressResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        search: str,
        max_results: Optional[int] = None,
        include_cities: Optional[str] = None,
        include_states: Optional[str] = None,
        include_zipcodes: Optional[str] = None,
        exclude_states: Optional[str] = None,
        prefer_cities: Optional[str] = None,
        prefer_states: Optional[str] = None,
        prefer_zipcodes: Optional[str] = None,
        prefer_ratio: Optional[int] = None,
        prefer_geolocation: Optional[str] = None,
        selected: Optional[str] = None,
        source: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetEnrichmentAddressResponse:
        r"""Fetch enriched address suggestions. Requires a partial address.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/profile-enrichment.read` scope.

        :param search: Partial or complete address to search.
        :param max_results: Maximum number of results to return.
        :param include_cities: Limits results to a list of given cities.
        :param include_states: Limits results to a list of given states.
        :param include_zipcodes: Limits results to a list of given zipcodes.
        :param exclude_states: Exclude list of states from results. No `include` pararmeters may be used with this parameter.
        :param prefer_cities: Display results with the listed cities at the top.
        :param prefer_states: Display results with the listed states at the top.
        :param prefer_zipcodes: Display results with the listed zipcodes at the top.
        :param prefer_ratio: Specifies the percentage of address suggestions that should be preferred and will appear at the top of the results.
        :param prefer_geolocation: If omitted or set to `city`, it uses the sender's IP address to determine location, then automatically adds the city and state
            to the preferCities value. This parameter takes precedence over other `include` or `exclude` parameters meaning that if it is
            not set to `none`, you may see addresses from areas you do not wish to see.
        :param selected: Useful for narrowing results with `addressLine2` suggestions such as `Apt` (denotes an apartment building with multiple residences).
        :param source: Include results from alternate data sources. Allowed values are `all` (non-postal addresses), or `postal` (postal addresses only).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetEnrichmentAddressRequest(
            search=search,
            max_results=max_results,
            include_cities=include_cities,
            include_states=include_states,
            include_zipcodes=include_zipcodes,
            exclude_states=exclude_states,
            prefer_cities=prefer_cities,
            prefer_states=prefer_states,
            prefer_zipcodes=prefer_zipcodes,
            prefer_ratio=prefer_ratio,
            prefer_geolocation=prefer_geolocation,
            selected=selected,
            source=source,
        )

        req = self._build_request_async(
            method="GET",
            path="/enrichment/address",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetEnrichmentAddressGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getEnrichmentAddress",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetEnrichmentAddressResponse(
                result=unmarshal_json_response(
                    components.EnrichedAddressResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productoptionvalidationerror import (
    ProductOptionValidationError,
    ProductOptionValidationErrorTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductOptionGroupValidationErrorTypedDict(TypedDict):
    name: NotRequired[str]
    description: NotRequired[str]
    min_select: NotRequired[str]
    max_select: NotRequired[str]
    options: NotRequired[Dict[str, ProductOptionValidationErrorTypedDict]]


class ProductOptionGroupValidationError(BaseModel):
    name: Optional[str] = None

    description: Optional[str] = None

    min_select: Annotated[Optional[str], pydantic.Field(alias="minSelect")] = None

    max_select: Annotated[Optional[str], pydantic.Field(alias="maxSelect")] = None

    options: Optional[Dict[str, ProductOptionValidationError]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["name", "description", "minSelect", "maxSelect", "options"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ProductOptionGroupValidationError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardtransactionfailurecode import CardTransactionFailureCode
from .cardtransactionstatus import CardTransactionStatus
from .transactionsource import TransactionSource
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CardTransactionDetailsTypedDict(TypedDict):
    r"""Card-specific details about the transaction."""

    status: NotRequired[CardTransactionStatus]
    r"""Status of a transaction within the card payment lifecycle."""
    failure_code: NotRequired[CardTransactionFailureCode]
    dynamic_descriptor: NotRequired[str]
    r"""An optional override of the default card statement descriptor for a transfer. Accounts must be enabled by Moov to set this field."""
    transaction_source: NotRequired[TransactionSource]
    r"""Specifies the nature and initiator of a transaction.

    Crucial for recurring and merchant-initiated transactions as per card scheme rules.
    Omit for customer-initiated e-commerce transactions.
    """
    initiated_on: NotRequired[datetime]
    confirmed_on: NotRequired[datetime]
    settled_on: NotRequired[datetime]
    failed_on: NotRequired[datetime]
    canceled_on: NotRequired[datetime]
    completed_on: NotRequired[datetime]
    interchange_qualification: NotRequired[str]
    r"""The program assigned by the card network that determines the interchange rate for the transfer."""
    fee_program: NotRequired[str]
    r"""The program assigned by the card network that determines the interchange rate for the transfer."""
    authorization_code: NotRequired[str]
    r"""An alphanumeric code generated by the card network to demonstrate approval for the transaction."""


class CardTransactionDetails(BaseModel):
    r"""Card-specific details about the transaction."""

    status: Optional[CardTransactionStatus] = None
    r"""Status of a transaction within the card payment lifecycle."""

    failure_code: Annotated[
        Optional[CardTransactionFailureCode], pydantic.Field(alias="failureCode")
    ] = None

    dynamic_descriptor: Annotated[
        Optional[str], pydantic.Field(alias="dynamicDescriptor")
    ] = None
    r"""An optional override of the default card statement descriptor for a transfer. Accounts must be enabled by Moov to set this field."""

    transaction_source: Annotated[
        Optional[TransactionSource], pydantic.Field(alias="transactionSource")
    ] = None
    r"""Specifies the nature and initiator of a transaction.

    Crucial for recurring and merchant-initiated transactions as per card scheme rules.
    Omit for customer-initiated e-commerce transactions.
    """

    initiated_on: Annotated[Optional[datetime], pydantic.Field(alias="initiatedOn")] = (
        None
    )

    confirmed_on: Annotated[Optional[datetime], pydantic.Field(alias="confirmedOn")] = (
        None
    )

    settled_on: Annotated[Optional[datetime], pydantic.Field(alias="settledOn")] = None

    failed_on: Annotated[Optional[datetime], pydantic.Field(alias="failedOn")] = None

    canceled_on: Annotated[Optional[datetime], pydantic.Field(alias="canceledOn")] = (
        None
    )

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    interchange_qualification: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="interchangeQualification",
        ),
    ] = None
    r"""The program assigned by the card network that determines the interchange rate for the transfer."""

    fee_program: Annotated[Optional[str], pydantic.Field(alias="feeProgram")] = None
    r"""The program assigned by the card network that determines the interchange rate for the transfer."""

    authorization_code: Annotated[
        Optional[str], pydantic.Field(alias="authorizationCode")
    ] = None
    r"""An alphanumeric code generated by the card network to demonstrate approval for the transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "status",
                "failureCode",
                "dynamicDescriptor",
                "transactionSource",
                "initiatedOn",
                "confirmedOn",
                "settledOn",
                "failedOn",
                "canceledOn",
                "completedOn",
                "interchangeQualification",
                "feeProgram",
                "authorizationCode",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CardTransactionDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class ListInvoicesValidationErrorData(BaseModel):
    status: Optional[str] = None
    customer_account_id: Annotated[
        Optional[str], pydantic.Field(alias="customerAccountID")
    ] = None
    created_start_date_time: Annotated[
        Optional[str], pydantic.Field(alias="createdStartDateTime")
    ] = None
    created_end_date_time: Annotated[
        Optional[str], pydantic.Field(alias="createdEndDateTime")
    ] = None
    due_start_date_time: Annotated[
        Optional[str], pydantic.Field(alias="dueStartDateTime")
    ] = None
    due_end_date_time: Annotated[
        Optional[str], pydantic.Field(alias="dueEndDateTime")
    ] = None
    skip: Optional[str] = None
    count: Optional[str] = None


@dataclass(unsafe_hash=True)
class ListInvoicesValidationError(MoovError):
    data: ListInvoicesValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: ListInvoicesValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .scheduledtransferlineitem import (
    ScheduledTransferLineItem,
    ScheduledTransferLineItemTypedDict,
)
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ScheduledTransferLineItemsTypedDict(TypedDict):
    r"""Line items for a scheduled transfer."""

    items: List[ScheduledTransferLineItemTypedDict]
    r"""The list of line items."""


class ScheduledTransferLineItems(BaseModel):
    r"""Line items for a scheduled transfer."""

    items: List[ScheduledTransferLineItem]
    r"""The list of line items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applebillingcontact import AppleBillingContact, AppleBillingContactTypedDict
from .linkapplepaytoken import LinkApplePayToken, LinkApplePayTokenTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LinkApplePayTypedDict(TypedDict):
    r"""The JSON structure returned from Apple Pay when authorizing a payment session.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypayment)
    for more information.
    """

    token: LinkApplePayTokenTypedDict
    r"""Contains the user's payment information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken)
    for more information.
    """
    billing_contact: NotRequired[AppleBillingContactTypedDict]
    r"""Billing contact information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
    for more information.
    """


class LinkApplePay(BaseModel):
    r"""The JSON structure returned from Apple Pay when authorizing a payment session.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypayment)
    for more information.
    """

    token: LinkApplePayToken
    r"""Contains the user's payment information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken)
    for more information.
    """

    billing_contact: Annotated[
        Optional[AppleBillingContact], pydantic.Field(alias="billingContact")
    ] = None
    r"""Billing contact information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
    for more information.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["billingContact"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    LinkApplePay.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .capabilityid import CapabilityID
from .capabilitystatus import CapabilityStatus
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookDataCapabilityUpdatedTypedDict(TypedDict):
    account_id: str
    capability_id: CapabilityID
    r"""Moov account capabilities.

    The `production-app`, `platform.production-app`, and / or `platform.wallet-transfers` capabilities might appear in your list. These are read-only capabilities that Moov requests and uses for account verification purposes. These capabilities remains active with your account and require no additional action.
    """
    status: CapabilityStatus
    r"""The status of the capability requested for an account."""
    foreign_id: NotRequired[str]


class WebhookDataCapabilityUpdated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    capability_id: Annotated[CapabilityID, pydantic.Field(alias="capabilityID")]
    r"""Moov account capabilities.

    The `production-app`, `platform.production-app`, and / or `platform.wallet-transfers` capabilities might appear in your list. These are read-only capabilities that Moov requests and uses for account verification purposes. These capabilities remains active with your account and require no additional action.
    """

    status: CapabilityStatus
    r"""The status of the capability requested for an account."""

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["foreignID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    WebhookDataCapabilityUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .achcreditsamedaypaymentmethod import (
    AchCreditSameDayPaymentMethod,
    AchCreditSameDayPaymentMethodTypedDict,
)
from .achcreditstandardpaymentmethod import (
    AchCreditStandardPaymentMethod,
    AchCreditStandardPaymentMethodTypedDict,
)
from .achdebitcollectpaymentmethod import (
    AchDebitCollectPaymentMethod,
    AchDebitCollectPaymentMethodTypedDict,
)
from .achdebitfundpaymentmethod import (
    AchDebitFundPaymentMethod,
    AchDebitFundPaymentMethodTypedDict,
)
from .applepaypaymentmethod import ApplePayPaymentMethod, ApplePayPaymentMethodTypedDict
from .cardpaymentpaymentmethod import (
    CardPaymentPaymentMethod,
    CardPaymentPaymentMethodTypedDict,
)
from .cardpresentpaymentpaymentmethod import (
    CardPresentPaymentPaymentMethod,
    CardPresentPaymentPaymentMethodTypedDict,
)
from .instantbankcreditpaymentmethod import (
    InstantBankCreditPaymentMethod,
    InstantBankCreditPaymentMethodTypedDict,
)
from .moovwalletpaymentmethod import (
    MoovWalletPaymentMethod,
    MoovWalletPaymentMethodTypedDict,
)
from .pullfromcardpaymentmethod import (
    PullFromCardPaymentMethod,
    PullFromCardPaymentMethodTypedDict,
)
from .pushtocardpaymentmethod import (
    PushToCardPaymentMethod,
    PushToCardPaymentMethodTypedDict,
)
from .rtpcreditpaymentmethod import (
    RtpCreditPaymentMethod,
    RtpCreditPaymentMethodTypedDict,
)
from moovio_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


PaymentMethodTypedDict = TypeAliasType(
    "PaymentMethodTypedDict",
    Union[
        MoovWalletPaymentMethodTypedDict,
        AchDebitFundPaymentMethodTypedDict,
        AchDebitCollectPaymentMethodTypedDict,
        AchCreditStandardPaymentMethodTypedDict,
        AchCreditSameDayPaymentMethodTypedDict,
        RtpCreditPaymentMethodTypedDict,
        CardPaymentPaymentMethodTypedDict,
        PushToCardPaymentMethodTypedDict,
        PullFromCardPaymentMethodTypedDict,
        ApplePayPaymentMethodTypedDict,
        CardPresentPaymentPaymentMethodTypedDict,
        InstantBankCreditPaymentMethodTypedDict,
    ],
)
r"""A method of moving money"""


PaymentMethod = Annotated[
    Union[
        Annotated[MoovWalletPaymentMethod, Tag("moov-wallet")],
        Annotated[AchDebitFundPaymentMethod, Tag("ach-debit-fund")],
        Annotated[AchDebitCollectPaymentMethod, Tag("ach-debit-collect")],
        Annotated[AchCreditStandardPaymentMethod, Tag("ach-credit-standard")],
        Annotated[AchCreditSameDayPaymentMethod, Tag("ach-credit-same-day")],
        Annotated[RtpCreditPaymentMethod, Tag("rtp-credit")],
        Annotated[CardPaymentPaymentMethod, Tag("card-payment")],
        Annotated[PushToCardPaymentMethod, Tag("push-to-card")],
        Annotated[PullFromCardPaymentMethod, Tag("pull-from-card")],
        Annotated[ApplePayPaymentMethod, Tag("apple-pay")],
        Annotated[CardPresentPaymentPaymentMethod, Tag("card-present-payment")],
        Annotated[InstantBankCreditPaymentMethod, Tag("instant-bank-credit")],
    ],
    Discriminator(
        lambda m: get_discriminator(m, "payment_method_type", "paymentMethodType")
    ),
]
r"""A method of moving money"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BankAccountStatus(str, Enum):
    NEW = "new"
    VERIFIED = "verified"
    VERIFICATION_FAILED = "verificationFailed"
    PENDING = "pending"
    ERRORED = "errored"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TokenType(str, Enum):
    r"""An [RFC 6750](https://www.rfc-editor.org/rfc/rfc6750#section-6.1) token type."""

    BEARER = "Bearer"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CardTransactionFailureCode(str, Enum):
    CALL_ISSUER = "call-issuer"
    DO_NOT_HONOR = "do-not-honor"
    PROCESSING_ERROR = "processing-error"
    INVALID_TRANSACTION = "invalid-transaction"
    INVALID_AMOUNT = "invalid-amount"
    NO_SUCH_ISSUER = "no-such-issuer"
    REENTER_TRANSACTION = "reenter-transaction"
    CVV_MISMATCH = "cvv-mismatch"
    LOST_OR_STOLEN = "lost-or-stolen"
    INSUFFICIENT_FUNDS = "insufficient-funds"
    INVALID_CARD_NUMBER = "invalid-card-number"
    INVALID_MERCHANT = "invalid-merchant"
    EXPIRED_CARD = "expired-card"
    INCORRECT_PIN = "incorrect-pin"
    TRANSACTION_NOT_ALLOWED = "transaction-not-allowed"
    SUSPECTED_FRAUD = "suspected-fraud"
    AMOUNT_LIMIT_EXCEEDED = "amount-limit-exceeded"
    VELOCITY_LIMIT_EXCEEDED = "velocity-limit-exceeded"
    REVOCATION_OF_AUTHORIZATION = "revocation-of-authorization"
    CARD_NOT_ACTIVATED = "card-not-activated"
    ISSUER_NOT_AVAILABLE = "issuer-not-available"
    COULD_NOT_ROUTE = "could-not-route"
    CARDHOLDER_ACCOUNT_CLOSED = "cardholder-account-closed"
    UNKNOWN_ISSUE = "unknown-issue"
    DUPLICATE_TRANSACTION = "duplicate-transaction"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from datetime import datetime
import httpx
from moovio_sdk.models.components import (
    amount as components_amount,
    refundcarddetails as components_refundcarddetails,
    refundstatus as components_refundstatus,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class CardAcquiringRefundData(BaseModel):
    refund_id: Annotated[str, pydantic.Field(alias="refundID")]
    r"""Identifier for the refund."""
    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]
    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]
    status: components_refundstatus.RefundStatus
    amount: components_amount.Amount
    card_details: Annotated[
        Optional[components_refundcarddetails.RefundCardDetails],
        pydantic.Field(alias="cardDetails"),
    ] = None


@dataclass(unsafe_hash=True)
class CardAcquiringRefund(MoovError):
    r"""Details of a card refund."""

    data: CardAcquiringRefundData = field(hash=False)

    def __init__(
        self,
        data: CardAcquiringRefundData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .individualname import IndividualName, IndividualNameTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .representativeresponsibilities import (
    RepresentativeResponsibilities,
    RepresentativeResponsibilitiesTypedDict,
)
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RepresentativeTypedDict(TypedDict):
    r"""Describes a business representative."""

    representative_id: str
    r"""Unique identifier for this representative."""
    name: IndividualNameTypedDict
    created_on: datetime
    updated_on: datetime
    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]
    address: NotRequired[AddressTypedDict]
    birth_date_provided: NotRequired[bool]
    r"""Indicates whether this representative's birth date has been provided."""
    government_id_provided: NotRequired[bool]
    r"""Indicates whether a government ID (SSN, ITIN, etc.) has been provided for this representative."""
    responsibilities: NotRequired[RepresentativeResponsibilitiesTypedDict]
    r"""Describes the job responsibilities of a business representative."""
    disabled_on: NotRequired[datetime]


class Representative(BaseModel):
    r"""Describes a business representative."""

    representative_id: Annotated[str, pydantic.Field(alias="representativeID")]
    r"""Unique identifier for this representative."""

    name: IndividualName

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    address: Optional[Address] = None

    birth_date_provided: Annotated[
        Optional[bool], pydantic.Field(alias="birthDateProvided")
    ] = None
    r"""Indicates whether this representative's birth date has been provided."""

    government_id_provided: Annotated[
        Optional[bool], pydantic.Field(alias="governmentIDProvided")
    ] = None
    r"""Indicates whether a government ID (SSN, ITIN, etc.) has been provided for this representative."""

    responsibilities: Optional[RepresentativeResponsibilities] = None
    r"""Describes the job responsibilities of a business representative."""

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "phone",
                "email",
                "address",
                "birthDateProvided",
                "governmentIDProvided",
                "responsibilities",
                "disabledOn",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Representative.model_rebuild()
except NameError:
    pass
